@@ -5,6 +5,8 @@ All models defined here are abstract, which means no tables are created in
 the feincms\_ namespace.
 """
 
+from functools import reduce
+import sys
 import operator
 import warnings
 
@@ -17,7 +19,7 @@ from django.db.models.loading import get_model
 from django.forms.widgets import Media
 from django.template.loader import render_to_string
 from django.utils.datastructures import SortedDict
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text, python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
 from feincms import ensure_completely_loaded
@@ -25,6 +27,7 @@ from feincms.extensions import ExtensionsMixin
 from feincms.utils import copy_model_instance
 
 
+@python_2_unicode_compatible
 class Region(object):
     """
     This class represents a region inside a template. Example regions might be
@@ -37,8 +40,8 @@ class Region(object):
         self.inherited = args and args[0] == 'inherited' or False
         self._content_types = []
 
-    def __unicode__(self):
-        return force_unicode(self.title)
+    def __str__(self):
+        return force_text(self.title)
 
     @property
     def content_types(self):
@@ -51,13 +54,14 @@ class Region(object):
                 for ct in self._content_types]
 
 
+@python_2_unicode_compatible
 class Template(object):
     """
     A template is a standard Django template which is used to render a
     CMS object, most commonly a page.
     """
 
-    def __init__(self, title, path, regions, key=None, preview_image=None):
+    def __init__(self, title, path, regions, key=None, preview_image=None, **kwargs):
         # The key is what will be stored in the database. If key is undefined
         # use the template path as fallback.
         if not key:
@@ -67,6 +71,10 @@ class Template(object):
         self.title = title
         self.path = path
         self.preview_image = preview_image
+        self.singleton = kwargs.get('singleton', False)
+        self.child_template = kwargs.get('child_template', None)
+        self.enforce_leaf = kwargs.get('enforce_leaf', False)
+        self.urlconf = kwargs.get('urlconf', None)
 
         def _make_region(data):
             if isinstance(data, Region):
@@ -76,8 +84,8 @@ class Template(object):
         self.regions = [_make_region(row) for row in regions]
         self.regions_dict = dict((r.key, r) for r in self.regions)
 
-    def __unicode__(self):
-        return force_unicode(self.title)
+    def __str__(self):
+        return force_text(self.title)
 
 
 class ContentProxy(object):
@@ -165,7 +173,7 @@ class ContentProxy(object):
         tmpl.append('GROUP BY region')
         tmpl = u' '.join(tmpl)
 
-        sql = ' UNION '.join([tmpl % (idx, cls._meta.db_table, pk)\
+        sql = ' UNION '.join([tmpl % (idx, cls._meta.db_table, pk)
             for idx, cls in enumerate(self.item._feincms_content_types)])
         sql = 'SELECT * FROM ( ' + sql + ' ) AS ct ORDER BY ct_idx'
 
@@ -222,7 +230,7 @@ class ContentProxy(object):
             self._cache['regions'] = dict((
                 region,
                 sorted(instances, key=lambda c: c.ordering),
-                ) for region, instances in contents.iteritems())
+                ) for region, instances in contents.items())
 
         return self._cache['regions']
 
@@ -383,10 +391,10 @@ def create_base_model(inherit_from=models.Model):
 
                 cls.template = property(_template)
 
-            cls.TEMPLATE_CHOICES = field._choices = [(
-                template.key,
-                template.title,
-                ) for template in cls._feincms_templates.values()]
+            cls.TEMPLATE_CHOICES = field._choices = [
+                (template.key, template.title,)
+                for template in cls._feincms_templates.values()
+            ]
             field.default = field.choices[0][0]
 
             # Build a set of all regions used anywhere
@@ -434,9 +442,11 @@ def create_base_model(inherit_from=models.Model):
                 app_label = cls._meta.app_label
                 ordering = ['ordering']
 
-            def __unicode__(self):
-                return u'%s on %s, ordering %s' % (
-                    self.region, self.parent, self.ordering)
+            def __str__(self):
+                return u'%s<pk=%s, parent=%s<pk=%s, %s>, region=%s, ordering=%d>' % (
+                    self.__class__.__name__, self.pk,
+                    self.parent.__class__.__name__, self.parent.pk, self.parent,
+                    self.region, self.ordering)
 
             def render(self, **kwargs):
                 """
@@ -498,7 +508,7 @@ def create_base_model(inherit_from=models.Model):
                 # from, therefore we ensure that the
                 # module is always known.
                 '__module__': cls.__module__,
-                '__unicode__': __unicode__,
+                '__str__': __str__,
                 'render': render,
                 'fe_render': fe_render,
                 'fe_identifier': fe_identifier,
@@ -510,8 +520,18 @@ def create_base_model(inherit_from=models.Model):
                 }
 
             # create content base type and save reference on CMS class
-            cls._feincms_content_model = type('%sContent' % cls.__name__,
-                (models.Model,), attrs)
+
+            name = '_Internal%sContentTypeBase' % cls.__name__
+            if hasattr(sys.modules[cls.__module__], name):
+                warnings.warn(
+                    'The class %s.%s has the same name as the class that '
+                    'FeinCMS auto-generates based on %s.%s. To avoid database'
+                    'errors and import clashes, rename one of these classes.'
+                    % (cls.__module__, name, cls.__module__, cls.__name__),
+                    RuntimeWarning)
+
+            cls._feincms_content_model = python_2_unicode_compatible(
+                type(name, (models.Model,), attrs))
 
             # list of concrete content types
             cls._feincms_content_types = []
@@ -636,6 +656,7 @@ def create_base_model(inherit_from=models.Model):
                 db_table = '%s_%s' % (cls._meta.db_table, class_name.lower())
                 verbose_name = model._meta.verbose_name
                 verbose_name_plural = model._meta.verbose_name_plural
+                permissions = model._meta.permissions
 
             attrs = {
                 # put the concrete content type into the
@@ -654,6 +675,8 @@ def create_base_model(inherit_from=models.Model):
                 (model, feincms_content_base,),
                 attrs)
             cls._feincms_content_types.append(new_type)
+            # For consistency's sake, also install the new type in the module
+            setattr(sys.modules[cls.__module__], class_name, new_type)
 
             if hasattr(getattr(new_type, 'process', None), '__call__'):
                 cls._feincms_content_types_with_process.append(new_type)
@@ -797,5 +820,6 @@ def create_base_model(inherit_from=models.Model):
 
     return Base
 
+
 # Legacy support
 Base = create_base_model()
