@@ -1,16 +1,11 @@
+from __future__ import print_function
+
+import re
 import sys
 
 from django.conf import settings as django_settings
 from django.http import Http404, HttpResponseRedirect
-
-
-def require_path_active_request_processor(page, request):
-    """
-    Checks whether any ancestors are actually inaccessible (ie. not
-    inactive or expired) and raise a 404 if so.
-    """
-    if not page.are_ancestors_active():
-        raise Http404()
+from django.utils.cache import add_never_cache_headers
 
 
 def redirect_request_processor(page, request):
@@ -25,6 +20,25 @@ def redirect_request_processor(page, request):
         raise Http404()
 
 
+def extra_context_request_processor(page, request):
+    """
+    Fills ``request._feincms_extra_context`` with a few useful variables.
+    """
+    request._feincms_extra_context.update({
+        # XXX This variable name isn't accurate anymore.
+        'in_appcontent_subpage': False,
+        'extra_path': '/',
+        })
+
+    url = page.get_absolute_url()
+    if request.path != url:
+        request._feincms_extra_context.update({
+            'in_appcontent_subpage': True,
+            'extra_path': re.sub('^' + re.escape(url.rstrip('/')), '',
+                request.path),
+            })
+
+
 def frontendediting_request_processor(page, request):
     """
     Sets the frontend editing state in the cookie depending on the
@@ -49,6 +63,17 @@ def frontendediting_request_processor(page, request):
     return response
 
 
+def frontendediting_response_processor(page, request, response):
+    # Add never cache headers in case frontend editing is active
+    if (hasattr(request, 'COOKIES')
+            and request.COOKIES.get('frontend_editing', False)):
+
+        if hasattr(response, 'add_post_render_callback'):
+            response.add_post_render_callback(add_never_cache_headers)
+        else:
+            add_never_cache_headers(response)
+
+
 def etag_request_processor(page, request):
     """
     Short-circuits the request-response cycle if the ETag matches.
@@ -111,7 +136,7 @@ def debug_sql_queries_response_processor(verbose=False, file=sys.stderr):
     Example::
 
         from feincms.module.page import models, processors
-        models.Page.register_response_procesors(
+        models.Page.register_response_processor(
             processors.debug_sql_queries_response_processor(verbose=True),
             )
     """
@@ -129,17 +154,18 @@ def debug_sql_queries_response_processor(verbose=False, file=sys.stderr):
             pass
 
         if verbose:
-            print >> file, "--------------------------------------------------------------"
+            print("--------------------------------------------------------------", file=file)
         time = 0.0
         i = 0
         for q in connection.queries:
             i += 1
             if verbose:
-                print >> file, "%d : [%s]\n%s\n" % ( i, q['time'], print_sql(q['sql']))
+                print("%d : [%s]\n%s\n" % (
+                    i, q['time'], print_sql(q['sql'])), file=file)
             time += float(q['time'])
 
-        print >> file, "--------------------------------------------------------------"
-        print >> file, "Total: %d queries, %.3f ms" % (i, time)
-        print >> file, "--------------------------------------------------------------"
+        print("--------------------------------------------------------------", file=file)
+        print("Total: %d queries, %.3f ms" % (i, time), file=file)
+        print("--------------------------------------------------------------", file=file)
 
     return processor
